==> pydraughts-0.6.1-py3-none-any.whl: draughts/engines/dxp_communication/dxp_run.py <==
# This file is an adaptation of DXC100_draughts_client (https://github.com/akalverboer/DXC100_draughts_client) by akalverboer.

import threading
import logging
from draughts.engines.dxp_communication.dxp_classes import DamExchange, MySocket, GameStatus, DXP_WHITE, DXP_BLACK
from draughts import Move
from typing import Union

last_move = None
accepted = None

logger = logging.getLogger("pydraughts")


class ConsoleHandler:
    def __init__(self) -> None:
        self.isRunning = False  # not used

    def run_command(self, comm: str) -> None:
        """Send a command to the DXP engine."""

        global current, mySock, lock
        global accepted, last_move
        logger.debug(f'comm {comm}')

        if comm.startswith('q') or comm.startswith('ex'):  # quit/exit
            logger.debug(f"Command terminate program: {comm.strip()}")
            # os._exit(1)  # does no cleanups

        elif comm.startswith('setup'):
            if current.started:
                logger.debug("Game started; setup not allowed")
                return
            if len(comm.split()) == 1:
                # Setup starting position
                logger.debug(f"Command setup starting position: {comm.strip()}")
                current = GameStatus()  # State(Position(board), C.WHITE)
            elif len(comm.split()) == 2:
                # Setup position with fen string (!!! without apostrophes and no spaces !!!)
                _, fen = comm.split(' ', 1)  # strip first word
                logger.debug("Command setup position with FEN string")
                logger.debug(f"FEN: {fen.strip()}")
                color = DXP_BLACK if fen[0] == 'B' else DXP_WHITE
                current = GameStatus(fen=fen, myColor=color)  # State(pos, color)
            elif len(comm.split()) == 3:
                # Setup position with fen string (!!! without apostrophes and no spaces !!!)
                _, fen, variant = comm.split(' ', 2)  # strip first word
                logger.debug("Command setup position with FEN and variant string")
                logger.debug(f"FEN: {fen.strip()} Variant: {variant}")
                color = DXP_BLACK if fen[0] == 'B' else DXP_WHITE
                current = GameStatus(fen=fen, myColor=color, variant=variant)  # State(pos, color)
            else:
                return

        elif comm.startswith('sm'):
            if current.started and current.myColor != current.get_color():
                logger.debug("Move not allowed; server has to move")
                return
            logger.debug(f"Command step move piece: {comm.strip()}")
            str_steps = comm.strip().split()[1].split('-')
            steps = list(map(int, str_steps))

            lock.acquire()  # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK
            if current.started and current.myColor == current.get_color():
                timeSpend = 0  # time spend for this move (future)

                board_move = []
                for index in range(1, len(steps)):
                    board_move.append([steps[index - 1], steps[index]])
                captures = []
                for move in current.pos.legal_moves():
                    if move.steps_move == steps:
                        captures = move.captures
                        break
                msg = dxp.msg_move(steps, captures, timeSpend)
                logger.debug(f'MOVE: {board_move}')

                try:
                    mySock.send(msg)
                    logger.debug(f"snd MOVE: {msg}")
                except Exception as err:
                    logger.debug(f"Error sending move: {err}")
                    return

                current.pos.push(Move(board_move=board_move))
            lock.release()  # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK

        elif comm.startswith('conn'):
            if mySock.sock is not None:
                logger.debug("Already connected")
                return
            if current.started:
                logger.debug("Game marked as started. First exit to start a new game.")
                return
            port: Union[str, int]
            host, port = '127.0.0.1', 27531  # default
            if len(comm.split()) == 2:
                _, host = comm.split()
            if len(comm.split()) == 3:
                _, host, port = comm.split()
            logger.debug(f"Command make connection with host {host} port {port}")
            try:
                mySock.open()
                mySock.connect(host, int(port))  # with timeout
                logger.debug(f"Successfully connected to remote host {host}, port {port}")
            except Exception as err:
                mySock.sock = None
                logger.debug(f"Error trying to connect: {err}")
                return

            if mySock.sock is not None:  # connected
                if not tReceiveHandler.isListening:
                    tReceiveHandler.start()

        elif comm.startswith('chat'):
            if len(comm.split()) == 1:
                return
            if len(comm.split()) > 1:
                _, txt = comm.split(' ', 1)  # strip first word
                txt = txt.strip()  # trim whitespace
                logger.debug(f"Command send chat message: {comm.strip()}")
                msg = dxp.msg_chat(txt)
                try:
                    mySock.send(msg)
                    logger.debug(f"snd CHAT: {msg}")
                except Exception as err:
                    logger.debug(f"Error sending chat message: {err}")
                    return

        elif comm.startswith('gamereq'):
            if current.started:
                logger.debug("Game already started; gamereq not allowed")
                return
            logger.debug(f"Command request new game: {comm.strip()}")
            last_move = None
            accepted = None
            myColor = "W"  # default
            gameTime = "120"  # default
            numMoves = "50"  # default
            if len(comm.split()) == 2:
                _, myColor = comm.split()
            if len(comm.split()) == 3:
                _, myColor, gameTime = comm.split()
            if len(comm.split()) == 4:
                _, myColor, gameTime, numMoves = comm.split()

            myColor = DXP_WHITE if myColor.upper().startswith('W') else DXP_BLACK
            current.myColor = myColor  # 0 or 1
            current.gameTime = gameTime
            current.numMoves = numMoves
            msg = dxp.msg_gamereq(myColor, gameTime, numMoves, current.pos, current.get_color())
            try:
                mySock.send(msg)
                logger.debug(f"snd GAMEREQ: {msg}")
            except Exception as err:
                logger.debug(f"Error sending game request: {err}")

        elif comm.startswith('gameend'):
            if not current.started:
                logger.debug("Game already finished; gameend not allowed")
                return
            logger.debug(f"Command finish game: {comm.strip()}")
            if current.started and current.myColor != current.get_color():
                logger.debug("Message gameend not allowed; wait until your turn")
                return
            if len(comm.split()) == 2:
                _, reason = comm.split()
            else:
                reason = "0"
            msg = dxp.msg_gameend(reason)

            try:
                mySock.send(msg)
                logger.debug(f"snd GAMEEND: {msg}")
            except Exception as err:
                logger.debug(f"Error sending gameend message: {err}")

        elif comm.startswith('backreq'):
            if not current.started:
                logger.debug("Game not started; backreq not allowed")
                return
            logger.debug("Not yet supported")

        else:
            logger.debug(f"Command unknown: {comm.strip()}")
            logger.debug(f"Unknown command, type h for help: {comm.strip()}")


class ReceiveHandler(threading.Thread):
    # Subslass of Thread to handle incoming messages from client.

    def __init__(self) -> None:
        threading.Thread.__init__(self)
        self.isListening = False

    def run(self) -> None:
        """Start the receiver. It receives the moves from the DXP engine."""
        # Handling incoming messages from server.
        # Excutes when thread started. Overriding python threading.Thread.run()

        logger.debug("ReceiveHandler started")
        global current, mySock, lock
        global accepted, last_move
        self.isListening = True
        logger.debug("DXP Client starts listening")
        while True:
            try:
                message = mySock.receive()  # wait for message
            except Exception as err:
                logger.debug(f"Error {err}")
                break

            lock.acquire()  # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK
            message = message[0:127]  # DXP max length
            dxpData = dxp.parse(message)
            if dxpData["type"] == "C":
                logger.debug(f"rcv CHAT: {message}")
                logger.debug(f"\nChat message: {dxpData['text']}")

            elif dxpData["type"] == "A":
                logger.debug(f"rcv GAMEACC: {message}")
                if dxpData["accCode"] == "0":
                    current.started = True
                    current.myColor = current.myColor  # as requested
                    current.engineName = dxpData["engineName"]
                    logger.debug(f"\nGame request accepted by {dxpData['engineName']}")
                    accepted = True
                else:
                    current.started = False
                    logger.debug(f"\nGame request NOT accepted by {dxpData['engineName']} Reason: {dxpData['accCode']}")
                    accepted = False

            elif dxpData["type"] == "E":
                logger.debug(f"rcv GAMEEND: {message}")
                logger.debug(f"\nRequest end of game accepted. Reason: {dxpData['reason']} Stop: {dxpData['stop']}")
                # Confirm game end by sending message back (if not sent by me)
                if current.started:
                    current.started = False
                    current.result = dxpData["reason"]
                    msg = dxp.msg_gameend(dxpData["reason"])
                    mySock.send(msg)
                    logger.debug(f"snd GAMEEND: {msg}")

            elif dxpData["type"] == "M":
                logger.debug(f"rcv MOVE: {message}")
                steps = [dxpData['from'], dxpData['to']]
                nsteps = list(map(int, steps))
                ntakes = list(map(int, dxpData['captures']))
                ntakes = list(map(lambda pos: str(pos).zfill(2), sorted(ntakes)))
                correct_move = None
                for move in current.pos.legal_moves():
                    if move.hub_position_move == f"{str(nsteps[0]).zfill(2)}{str(nsteps[-1]).zfill(2)}{''.join(ntakes)}":
                        correct_move = move

                if correct_move is not None:
                    last_move = correct_move
                    logger.debug(f"\nMove received: {correct_move.steps_move}")
                    current.pos.push(correct_move)
                else:
                    logger.debug(f"Error: received move is illegal [{message}]")

            elif dxpData["type"] == "B":
                # For the time being do not confirm request from server: send message back.
                logger.debug(f"rcv BACKREQ: {message}")
                accCode = "1"  # 0: BACK YES; 1: BACK NO; 2: CONTINUE
                msg = dxp.msg_backacc(accCode)
                mySock.send(msg)
                logger.debug(f"snd BACKACC: {msg}")

            elif dxpData["type"] == "K":
                # Answer to my request to move back
                logger.debug(f"rcv BACKACC: {message}")
                logger.debug(f"rcv BACKACC: {message}")  # TEST
                accCode = dxpData['accCode']
                if accCode == "0":
                    # Actions to go back in history as specified in my request
                    raise NotImplementedError("Moving back is not implemented")

            else:
                logger.debug(f"rcv UNKNOWN: {message}")
                logger.debug(f"\nrcv Unknown message: {message}")

            lock.release()  # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK

        self.isListening = False
        logger.error("Listening stopped; connection broken")
        logger.debug("Connection broken; receiveHandler stopped. ")
        logger.debug("Save your data and exit program to start again. ")
        return None


dxp = DamExchange()  # global, singleton
mySock = MySocket()  # global, singleton
current = GameStatus()  # global; use default parms
lock = threading.Lock()  # global

# use 1 thread to listen to incoming messages
tConsoleHandler = ConsoleHandler()  # Thread subclass instance
tReceiveHandler = ReceiveHandler()  # Thread subclass instance. Start when connected

==> pydraughts-0.6.3-py3-none-any.whl: draughts/engines/dxp_communication/dxp_run.py <==
# This file is an adaptation of DXC100_draughts_client (https://github.com/akalverboer/DXC100_draughts_client) by akalverboer.

import threading
import logging
from draughts.engines.dxp_communication.dxp_classes import DamExchange, MySocket, GameStatus, DXP_WHITE, DXP_BLACK
from draughts import Move
from typing import Union

last_move = None
last_move_changed = False
accepted = None
gameend_sent = False

logger = logging.getLogger("pydraughts")


class ConsoleHandler:
    def __init__(self) -> None:
        self.isRunning = False  # not used

    def run_command(self, comm: str) -> None:
        """Send a command to the DXP engine."""

        global current, mySock, lock
        global accepted, last_move, last_move_changed, gameend_sent
        logger.debug(f'comm {comm}')

        if comm.startswith('q') or comm.startswith('ex'):  # quit/exit
            logger.debug(f"Command terminate program: {comm.strip()}")
            # os._exit(1)  # does no cleanups

        elif comm.startswith('setup'):
            if current.started:
                logger.debug("Game started; setup not allowed")
                return
            if len(comm.split()) == 1:
                # Setup starting position
                logger.debug(f"Command setup starting position: {comm.strip()}")
                current = GameStatus()  # State(Position(board), C.WHITE)
            elif len(comm.split()) == 2:
                # Setup position with fen string (!!! without apostrophes and no spaces !!!)
                _, fen = comm.split(' ', 1)  # strip first word
                logger.debug("Command setup position with FEN string")
                logger.debug(f"FEN: {fen.strip()}")
                color = DXP_BLACK if fen[0] == 'B' else DXP_WHITE
                current = GameStatus(fen=fen, myColor=color)  # State(pos, color)
            elif len(comm.split()) == 3:
                # Setup position with fen string (!!! without apostrophes and no spaces !!!)
                _, fen, variant = comm.split(' ', 2)  # strip first word
                logger.debug("Command setup position with FEN and variant string")
                logger.debug(f"FEN: {fen.strip()} Variant: {variant}")
                color = DXP_BLACK if fen[0] == 'B' else DXP_WHITE
                current = GameStatus(fen=fen, myColor=color, variant=variant)  # State(pos, color)
            else:
                return

        elif comm.startswith('sm'):
            if current.started and current.myColor != current.get_color():
                logger.debug("Move not allowed; server has to move")
                return
            logger.debug(f"Command step move piece: {comm.strip()}")
            str_steps = comm.strip().split()[1].split('-')
            steps = list(map(int, str_steps))

            lock.acquire() # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK
            if current.started and current.myColor == current.get_color():
                timeSpend = 0  # time spend for this move (future)

                board_move = []
                for index in range(1, len(steps)):
                    board_move.append([steps[index - 1], steps[index]])
                captures = []
                for move in current.pos.legal_moves():
                    if move.steps_move == steps:
                        captures = move.captures
                        break
                last_move_changed = False
                msg = dxp.msg_move(steps, captures, timeSpend)
                logger.debug(f'MOVE: {board_move}')

                try:
                    mySock.send(msg)
                    logger.debug(f"snd MOVE: {msg}")
                except Exception as err:
                    logger.debug(f"Error sending move: {err}")
                    return

                current.pos.push(Move(board_move=board_move))
            lock.release() # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK

        elif comm.startswith('conn'):
            if mySock.sock is not None:
                logger.debug("Already connected")
                return
            if current.started:
                logger.debug("Game marked as started. First exit to start a new game.")
                return
            port: Union[str, int]
            host, port = '127.0.0.1', 27531  # default
            if len(comm.split()) == 2:
                _, host = comm.split()
            if len(comm.split()) == 3:
                _, host, port = comm.split()
            logger.debug(f"Command make connection with host {host} port {port}")
            try:
                mySock.open()
                mySock.connect(host, int(port))  # with timeout
                logger.debug(f"Successfully connected to remote host {host}, port {port}")
            except Exception as err:
                mySock.sock = None
                logger.debug(f"Error trying to connect: {err}")
                return

            if mySock.sock is not None:  # connected
                if not tReceiveHandler.isListening:
                    tReceiveHandler.start()

        elif comm.startswith("disconn"):
            try:
                mySock.close()
                logger.debug(f"Successfully closed socket.")
            except Exception as err:
                mySock.sock = None
                logger.debug(f"Error trying to close socket: {err}")
                return

        elif comm.startswith('chat'):
            if len(comm.split()) == 1:
                return
            if len(comm.split()) > 1:
                _, txt = comm.split(' ', 1)  # strip first word
                txt = txt.strip()  # trim whitespace
                logger.debug(f"Command send chat message: {comm.strip()}")
                msg = dxp.msg_chat(txt)
                try:
                    mySock.send(msg)
                    logger.debug(f"snd CHAT: {msg}")
                except Exception as err:
                    logger.debug(f"Error sending chat message: {err}")
                    return

        elif comm.startswith('gamereq'):
            if current.started:
                logger.debug("Game already started; gamereq not allowed")
                return
            logger.debug(f"Command request new game: {comm.strip()}")
            last_move = None
            last_move_changed = False
            accepted = None
            gameend_sent = False
            myColor = "W"  # default
            gameTime = "120"  # default
            numMoves = "50"  # default
            if len(comm.split()) == 2:
                _, myColor = comm.split()
            if len(comm.split()) == 3:
                _, myColor, gameTime = comm.split()
            if len(comm.split()) == 4:
                _, myColor, gameTime, numMoves = comm.split()

            myColor = DXP_WHITE if myColor.upper().startswith('W') else DXP_BLACK
            current.myColor = myColor  # 0 or 1
            current.gameTime = gameTime
            current.numMoves = numMoves
            msg = dxp.msg_gamereq(myColor, gameTime, numMoves, current.pos, current.get_color())
            try:
                mySock.send(msg)
                logger.debug(f"snd GAMEREQ: {msg}")
            except Exception as err:
                logger.debug(f"Error sending game request: {err}")

        elif comm.startswith('gameend'):
            if not current.started:
                logger.debug("Game already finished; gameend not allowed")
                return
            logger.debug(f"Command finish game: {comm.strip()}")
            if current.started and current.myColor != current.get_color():
                logger.debug("Message gameend not allowed; wait until your turn")
                return
            if len(comm.split()) == 2:
                _, reason = comm.split()
            else:
                reason = "0"
            accepted = None
            gameend_sent = True
            msg = dxp.msg_gameend(reason)

            try:
                mySock.send(msg)
                logger.debug(f"snd GAMEEND: {msg}")
            except Exception as err:
                logger.debug(f"Error sending gameend message: {err}")

        elif comm.startswith('backreq'):
            if not current.started:
                logger.debug("Game not started; backreq not allowed")
                return
            logger.debug("Not yet supported")

        else:
            logger.debug(f"Command unknown: {comm.strip()}")
            logger.debug(f"Unknown command, type h for help: {comm.strip()}")


class ReceiveHandler(threading.Thread):
    # Subslass of Thread to handle incoming messages from client.

    def __init__(self) -> None:
        threading.Thread.__init__(self)
        self.isListening = False

    def run(self) -> None:
        """Start the receiver. It receives the moves from the DXP engine."""
        # Handling incoming messages from server.
        # Excutes when thread started. Overriding python threading.Thread.run()

        logger.debug("ReceiveHandler started")
        global current, mySock, lock
        global accepted, last_move, last_move_changed, gameend_sent
        self.isListening = True
        logger.debug("DXP Client starts listening")
        while True:
            try:
                message = mySock.receive()  # wait for message
            except Exception as err:
                logger.debug(f"Error {err}")
                mySock.close()
                break

            lock.acquire()  # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK
            message = message[0:127]  # DXP max length
            dxpData = dxp.parse(message)
            if dxpData["type"] == "C":
                logger.debug(f"rcv CHAT: {message}")
                logger.debug(f"\nChat message: {dxpData['text']}")

            elif dxpData["type"] == "A":
                logger.debug(f"rcv GAMEACC: {message}")
                if dxpData["accCode"] == "0":
                    current.myColor = current.myColor  # as requested
                    current.engineName = dxpData["engineName"]
                    logger.debug(f"\nGame request accepted by {dxpData['engineName']}")
                    accepted = True
                    gameend_sent = False
                    current.started = True
                else:
                    logger.debug(f"\nGame request NOT accepted by {dxpData['engineName']} Reason: {dxpData['accCode']}")
                    accepted = False
                    current.started = False

            elif dxpData["type"] == "E":
                logger.debug(f"rcv GAMEEND: {message}")
                logger.debug(f"\nRequest end of game accepted. Reason: {dxpData['reason']} Stop: {dxpData['stop']}")
                # Confirm game end by sending message back (if not sent by me)
                if current.started and not gameend_sent:
                    current.result = dxpData["reason"]
                    msg = dxp.msg_gameend(dxpData["reason"])
                    mySock.send(msg)
                    logger.debug(f"snd GAMEEND: {msg}")
                    gameend_sent = True
                    current.started = False

            elif dxpData["type"] == "M":
                logger.debug(f"rcv MOVE: {message}")
                steps = [dxpData['from'], dxpData['to']]
                nsteps = list(map(int, steps))
                ntakes = list(map(int, dxpData['captures']))
                ntakes = list(map(lambda pos: str(pos).zfill(2), sorted(ntakes)))
                correct_move = None
                for move in current.pos.legal_moves():
                    if move.hub_position_move == f"{str(nsteps[0]).zfill(2)}{str(nsteps[-1]).zfill(2)}{''.join(ntakes)}":
                        correct_move = move

                if correct_move is not None:
                    last_move = correct_move
                    logger.debug(f"Move received: {correct_move.steps_move}")
                    current.pos.push(correct_move)
                    last_move_changed = True
                else:
                    logger.debug(f"Error: received move is illegal [{message}]")

            elif dxpData["type"] == "B":
                # For the time being do not confirm request from server: send message back.
                logger.debug(f"rcv BACKREQ: {message}")
                accCode = "1"  # 0: BACK YES; 1: BACK NO; 2: CONTINUE
                msg = dxp.msg_backacc(accCode)
                mySock.send(msg)
                logger.debug(f"snd BACKACC: {msg}")

            elif dxpData["type"] == "K":
                # Answer to my request to move back
                logger.debug(f"rcv BACKACC: {message}")
                logger.debug(f"rcv BACKACC: {message}")  # TEST
                accCode = dxpData['accCode']
                if accCode == "0":
                    # Actions to go back in history as specified in my request
                    raise NotImplementedError("Moving back is not implemented")

            else:
                logger.debug(f"rcv UNKNOWN: {message}")
                logger.debug(f"\nrcv Unknown message: {message}")

            lock.release()  # LOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCKLOCK

        self.isListening = False
        if mySock.closed:
            logger.debug("Listening stopped; connection broken")
        else:
            logger.error("Listening stopped; connection broken")
        logger.debug("Connection broken; receiveHandler stopped. ")
        logger.debug("Save your data and exit program to start again. ")
        return None


dxp = DamExchange()  # global, singleton
mySock = MySocket()  # global, singleton
current = GameStatus()  # global; use default parms
lock = threading.Lock()  # global

# use 1 thread to listen to incoming messages
tConsoleHandler = ConsoleHandler()  # Thread subclass instance
tReceiveHandler = ReceiveHandler()  # Thread subclass instance. Start when connected
